# functions mixtures
CA_complete2_Min<-function(C_mat, Min, Max, Slopes, Ec50s, a=0, b=0, interact="none", multicore=FALSE, mc.cores=4){
  param<-data.frame(Slopes, Ec50s)
  if (interact=="none"){
    a<-0
    b<-0
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)  # C1 corresponding to Y 
      G<-(sum(x / ecs))-1
      return(abs(G))
    }
  }
  else if (interact=="SA"){
    b<-0
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)  # C1 corresponding to Y 
      G<-(sum(x / ecs))-exp(a*prod(x/param$Ec50s/(sum(x/param$Ec50s))))
      return(abs(G))
    }
  }
  else if (interact=="DR"){
    if (length(b)!=(dim(C_mat)[2]-1))
      stop("length of b should be equal to the number of chemicals -1.")
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)  # Cs corresponding to Y 
      G<-(sum(x / ecs))-exp((a+b%*%((x/param$Ec50s/(sum(x/param$Ec50s))))[1:(dim(C_mat)[2]-1)])*prod(x/param$Ec50s/(sum(x/param$Ec50s))))
      return(abs(G))
    }
  }
  else if (interact=="DL"){
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      if (length(b)!=(dim(C_mat)[2]-1))
        stop("length of b should be equal to the number of chemicals -1.")
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)  # C1 corresponding to Y 
      G<-(sum(x / ecs))-exp(a*(1-b*(sum(x/param$Ec50s)))*prod(x/param$Ec50s/(sum(x/param$Ec50s))))
      return(abs(G))
    }
  }
  else 
    stop("please specify interaction model")
  Y_f<-function(x, Min=Min, Max=Max, param=param, a=a, b=b) {
    if (all(x==0)){          #no chemical
      if  (all(param$Slopes > 0))
        return(Max)
      if  (all(param$Slopes < 0))
        return(Min)
    }
    #Y<-optimize(f=function(Y) f(Y=Y, x=x, Max=Max, param=param, a=a, b=b), interval=c(0,Max*1.2))$minimum
    Y<-optimize(f=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), interval=c(Min,Max))$minimum
    return(Y)
  }
  res_CA<-apply(C_mat,1, function(x) Y_f(x, Min=Min, Max=Max, param=param, a=a, b=b))
  return(res_CA)
} 

CA_complete2_RSS_Min<-function(C_mat, Response, Min, Max, Slopes, Ec50s, a=0, b=0, interact="none", multicore=FALSE, mc.cores=4){
  if (length(Response)!=(dim(C_mat)[1]))
    stop("Should be as many responses as there are conditions")
  res_CA<-CA_complete2_Min(C_mat=C_mat, Min=Min, Max=Max, Slopes=Slopes, Ec50s=Ec50s, a=a, b=b, interact=interact, multicore=multicore, mc.cores=mc.cores)
  res_CA_RSS<-sum((res_CA-Response)^2)
  return(res_CA_RSS)
}



CA_complete2_fit_Min_speed<-function(C_mat, Response, param=NULL, upper=NULL, lower=NULL, start=NULL, interact="none", identical_slopes=FALSE, iter=500, multicore=FALSE, mc.cores=4){
  if (!is.null(param)){
    if ((is.null(param$Min))|(is.null(param$Max))|(is.null(param$Slopes))|(is.null(param$Ec50s)))
      stop("param misspecification")
    if (interact=="none")
      stop("please specify interaction")
    if (interact=="SA"){
      if (missing(upper))
        upper=20  # the intervals for a and b must be larger beacuse they can compensate each other
      if (missing(lower))
        lower=-20
      res_CA_optim<-optimize(f=function(x){
        CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=param$Min, Max=param$Max, Slopes=param$Slopes, Ec50s=param$Ec50s, a=x, interact=interact, multicore=multicore, mc.cores=mc.cores)
      }, upper=upper, lower=lower)
      return(list(a=res_CA_optim$minimum, RSS=res_CA_optim$objective))
    }
    if (interact=="DR"){
      res_CA_optim<-optim(par=rep(0, dim(C_mat)[2]), fn=function(x){
        CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=param$Min, Max=param$Max, Slopes=param$Slopes, Ec50s=param$Ec50s, a=x[1], b=x[2:length(x)], interact=interact, multicore=multicore, mc.cores=mc.cores)
      })
      cat(res_CA_optim$convergence)
      return(list(a=res_CA_optim$par[1], b=res_CA_optim$par[2: (2+(dim(C_mat)[2]-1)-1)], RSS=res_CA_optim$value))
    }
    if (interact=="DL"){
      res_CA_optim<-optim(par=c(0,0), fn=function(x){
        CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=param$Min, Max=param$Max, Slopes=param$Slopes, Ec50s=param$Ec50s, a=x[1], b=x[2], interact=interact, multicore=multicore, mc.cores=mc.cores)
      })
      cat(res_CA_optim$convergence)
      return(list(a=res_CA_optim$par[1], b=res_CA_optim$par[2], RSS=res_CA_optim$value))
    }
  }
  if (is.null(param)){
    require(dfoptim)
    if (any(C_mat==0))
      mean_C_mat<-exp(apply(log(C_mat[-which(C_mat==0, arr.ind=TRUE)[,1],]),2,mean))
    else
      mean_C_mat<-exp(apply(log(C_mat),2,mean))
    if (!identical_slopes){
      if (interact=="none"){
        if (missing(upper))
          upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10)
        if (missing(lower))
          lower=c(min(Response)/5,0,-100,-100,0,0)
        if (missing(start))
          start=c(max(Response),-1,-1,mean_C_mat)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), RSS=res_CA_nmk$value))
      }
      if (interact=="SA"){
        if (missing(upper))
          upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10, 20)  # the intervals for a and b must be larger beacuse they can compensate each other
        if (missing(lower))
          lower=c(min(Response)/5,0,-100,-100,0,0, -20)
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), a=res_CA_nmk$par[7], RSS=res_CA_nmk$value))
      }
      if (interact=="DR"){
        if (missing(upper))
          upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10, 20, rep(30, (dim(C_mat)[2]-1)))  # the intervals for a and b must be larger beacuse they can compensate each other
        if (missing(lower))
          lower=c(min(Response)/5, 0,-100,-100,0,0, -20, rep(-30, (dim(C_mat)[2]-1)))
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0,rep(0, (dim(C_mat)[2]-1)))
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], b=x[8:length(x)], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), a=res_CA_nmk$par[7], b=res_CA_nmk$par[8: (8+(dim(C_mat)[2]-1)-1)], RSS=res_CA_nmk$value))
      }
      if (interact=="DL"){
        if (missing(upper))
          upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10, 20,30)
        if (missing(lower))
          lower=c(min(Response)/5, 0,-100,-100,0,0, -20, -30)
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], b=x[8], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), a=res_CA_nmk$par[7], b=res_CA_nmk$par[8], RSS=res_CA_nmk$value))
      }
    }
    else {
      if (interact=="none"){
        if (missing(upper))
          upper=c(max(Response), max(Response)*5, 0, mean_C_mat*10)
        if (missing(lower))
          lower=c(min(Response)/5, 0,-100,0,0)
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3], res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), RSS=res_CA_nmk$value))
      }
      if (interact=="SA"){
        if (missing(upper))
          upper=c(max(Response), max(Response)*5, 0, mean_C_mat*10, 20)  # the intervals for a and b must be larger beacuse they can compensate each other
        if (missing(lower))
          lower=c(min(Response)/5, 0,-100,0,0, -20)
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3], res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), a=res_CA_nmk$par[6], RSS=res_CA_nmk$value))
      }
      if (interact=="DR"){
        if (missing(upper))
          upper=c(max(Response), max(Response)*5, 0,mean_C_mat*10, 20, rep(30, (dim(C_mat)[2]-1)))  # the intervals for a and b must be larger beacuse they can compensate each other
        if (missing(lower))
          lower=c(min(Response)/5, 0,-100,0,0, -20, rep(-30, (dim(C_mat)[2]-1)))
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0, rep(0, (dim(C_mat)[2]-1)))
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], b=x[7:length(x)], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3], res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), a=res_CA_nmk$par[6], b=res_CA_nmk$par[7: (7+(dim(C_mat)[2]-1)-1)], RSS=res_CA_nmk$value))
      }
      if (interact=="DL"){
        if (missing(upper))
          upper=c(max(Response),max(Response)*5, 0,mean_C_mat*10, 20,30)
        if (missing(lower))
          lower=c(min(Response)/5,0,-100,0,0, -20, -30)
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], b=x[7], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3], res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), a=res_CA_nmk$par[6], b=res_CA_nmk$par[7], RSS=res_CA_nmk$value))
      }
    }
  }
}


CA_complete2_Min_modified_nlm<-function(C_mat, Min, Max, Slopes, Ec50s, a=0, b=0, interact="none", multicore=FALSE, mc.cores=4){    # diff?rent de la feuille Excel de Jonker. Tient compte du fait que les pentes des substances seules peuvent ?tre diff?rentes : la d?viation n'est plus calcul?e par rapport ? l'EC50. En pratique ?a ne fonctionne pas. probl?me d'optimisation.  J'ai mis un garde fou pour ne pas pr?dire de r?ponses trop basses, mais ?a ne fonctionne pas et ?a embp^che la synergie
  param<-data.frame(Slopes, Ec50s)
  if (interact=="none"){
    a<-0
    b<-0
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)   
      G<-(sum(x / ecs))-1
      return(abs(G))
    }
    Y_f<-function(x, Min=Min, Max=Max, param=param, a=a, b=b) {
      if (all(x==0)){          #no chemical
        if  (all(param$Slopes > 0))
          return(Max)
        if  (all(param$Slopes < 0))
          return(Min)
      }
      Y<-optimize(f=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), interval=c(Min,Max))$minimum
      return(Y)
    }
  }
  else if (interact=="SA"){
    b<-0
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)   
      G<-(sum(x / ecs))-exp(a*prod(x/ecs/(sum(x/ecs))))   #prod(x/ecs/(sum(x/ecs))) est ?lev?e si le m?lange est ?quilibr? en ?quivalents toxiques.
      return(abs(G))
      #return(G^2)
    }
    Y_f<-function(x, Min=Min, Max=Max, param=param, a=a, b=b) {
      if (all(x==0)){          #no chemical
        if  (all(param$Slopes > 0))
          return(Max)
        if  (all(param$Slopes < 0))
          return(Min)
      }
      else {
        Y<-optimize(f=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), interval=c(Min,Max*1.2))$minimum 
        if ((all(x!=0))&&((prod(x/param$Ec50s/(sum(x/param$Ec50s)))<0.16)|(Y<(Min+(Max-Min)/20))|(Y>(Min+(Max-Min)*0.95))|(sum(x/param$Ec50s)<0.1)|(sum(x/param$Ec50s)>0.9))){# 0.16 ar cela correspond à 20%80% de contribution dans l'hypothèse de pentes égales
          #p_vect<-c(Max/10/c(1000000, 500000, 300000, 200000, 100000, 30000, 10000, 3000, 1000,  300, 100, 20),Max*c(0.95, 0.99,0.995,0.999, 0.9995,0.9999,0.99995,0.99999,0.999993,0.999996,0.999999), Max/2) #OK
          #p_vect<-c(Max/c(10000000, 3000000, 1000000, 300000, 100000, 30000, 10000, 1000, 30),Max*(1-1/c(10000000, 3000000, 1000000, 300000, 100000, 30000, 10000, 1000, 30)), Max/2) #OK
          p_vect<-c(Min+(Max-Min)/c(10000000,  1000000, 100000, 10000, 1000, 100,10,3),Min+(Max-Min)*(1-1/c(10000000, 1000000, 100000, 10000, 1000,  100,10,3)))
          #res_nlm<-lapply(p_vect, function(p) nlm(f=function(Y) f(Y=Y, x=x, Max=Max, param=param, a=a, b=b), p=p, iterlim=10000)) # 3 fois moins long qu'un seul DEoptim
          res_nlminb<-lapply(p_vect, function(p) nlminb(objective=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), start=p, upper=Max, lower=Min)) # aussi long qu'un seul Deoptim
          res_nlm_min<-unlist(lapply(res_nlminb, function(x) x$objective))
          Y<-res_nlminb[[which(res_nlm_min==min(res_nlm_min))[1]]]$par
        }
        return(Y)
      }
    }
  }
  else if (interact=="DR"){
    if (length(b)!=(dim(C_mat)[2]-1))
      stop("length of b should be equal to the number of chemicals -1.")
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)  # Cs corresponding to Y 
      G<-(sum(x / ecs))-exp((a+b%*%((x/ecs/(sum(x/ecs)))[1:(dim(C_mat)[2]-1)]))*prod(x/ecs/(sum(x/ecs)))) #the closest to Jonker
      #G<-(sum(x / ecs))-exp(a*(1+b%*%((x/ecs/(sum(x/ecs)))[1:(dim(C_mat)[2]-1)]))*prod(x/ecs/(sum(x/ecs))))   # doesn't fit very well
      #G<-(sum(x / ecs))-exp((b%*%((x/ecs)[1:(dim(C_mat)[2]-1)]))*prod(x/ecs/(sum(x/ecs))))
      #return(abs(G)-1000*sign(Y-min(Max/(1+(x/param$Ec50s)^param$Slopes))))  # contrainte: la r?ponse doit ?tre sup?rieure au plus fort effet provoqu? par les substances seules   # interdit un antogonisme tr?s prononc?.
      #return(abs(G)+Max*exp(-(Y-min(Max/(1+(x/param$Ec50s)^param$Slopes)))))    # ?a encourage un Y ?lev?.
      return(abs(G))
    }
    Y_f<-function(x, Min=Min, Max=Max, param=param, a=a, b=b) {
      if (all(x==0)){          #no chemical
        if  (all(param$Slopes > 0))
          return(Max)
        if  (all(param$Slopes < 0))
          return(Min)
      }
      else {
        Y<-optimize(f=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), interval=c(Min,Max))$minimum #version initiale 
        # ecs_final <- param$Ec50s * ((Max - Y)/Y) ^ (1 / param$Slopes)
        if ((all(x!=0))&&(long)&&((prod(x/param$Ec50s/(sum(x/param$Ec50s)))<0.16)|(Y<(Min+(Max-Min)/20))|(Y>(Min+(Max-Min)*0.95))|(sum(x/param$Ec50s)<0.1))){
          #p_vect<-c(Max/10/c(1000000, 500000, 300000, 200000, 100000, 30000, 10000, 3000, 1000,  300, 100, 20),Max*c(0.95, 0.99,0.995,0.999, 0.9995,0.9999,0.99995,0.99999,0.999993,0.999996,0.999999), Max/2)
          p_vect<-c(Min+(Max-Min)/c(10000000, 3000000, 1000000, 300000, 100000, 30000, 10000, 1000, 100,10,3),Min+(Max-Min)*(1-1/c(10000000, 3000000, 1000000, 300000, 100000, 30000, 10000, 1000, 100,10,3)))
          #res_nlm<-lapply(p_vect, function(p) nlm(f=function(Y) f(Y=Y, x=x, Max=Max, param=param, a=a, b=b), p=p, iterlim=10000)) # 3 fois moins long qu'un seul DEoptim
          res_nlminb<-lapply(p_vect, function(p) nlminb(objective=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), start=p, lower=Min, upper=Max)) # aussi long qu'un seul Deoptim
          res_nlm_min<-unlist(lapply(res_nlminb, function(x) x$objective))
          Y<-res_nlminb[[which(res_nlm_min==min(res_nlm_min))[1]]]$par
        }
        return(Y)
      }
    }
  }
  else if (interact=="DL"){
    f<-function(Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b){
      if (length(b)!=(dim(C_mat)[2]-1))
        stop("length of b should be equal to the number of chemicals -1.")
      ecs <- param$Ec50s * ((Max - Y)/(Y - Min)) ^ (1 / param$Slopes)  # C1 corresponding to Y 
      G<-(sum(x / ecs))-exp(a*(1-b*(sum(x/param$Ec50s)))*prod(x/ecs/(sum(x/ecs))))  # closest to Jonker
      #G<-(sum(x / ecs))-exp((a-b*(sum(x/ecs)))*prod(x/ecs/(sum(x/ecs))))    # should be easier to optimize
      return(abs(G))
    }
    Y_f<-function(x, Min=Min, Max=Max, param=param, a=a, b=b) {
      if (all(x==0)){          #no chemical
        if  (all(param$Slopes > 0))
          return(Max)
        if  (all(param$Slopes < 0))
          return(Min)
      }
      else {
        Y<-optimize(f=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), interval=c(Min,Max))$minimum #version initiale 
        # ecs_final <- param$Ec50s * ((Max - Y)/Y) ^ (1 / param$Slopes)
        #p_vect<-c(Max/10/c(1000000, 500000, 300000, 200000, 100000, 30000, 10000, 3000, 1000,  300, 100, 20),Max*c(0.95, 0.99,0.995,0.999, 0.9995,0.9999,0.99995,0.99999,0.999993,0.999996,0.999999), Max/2)
        if ((all(x!=0))&&((prod(x/param$Ec50s/(sum(x/param$Ec50s)))<0.2)|(Y<(Max/20))|(Y>(Max*0.95))|(sum(x/param$Ec50s)<0.1)|(sum(x/param$Ec50s)>0.9))){
          p_vect<-c(Min+(Max-Min)/c(10000000, 3000000, 1000000, 300000, 100000, 30000, 10000, 1000, 100,10,3),Min+(Max-Min)*(1-1/c(10000000, 3000000, 1000000, 300000, 100000, 30000, 10000, 1000, 100,10,3)))
          #res_nlm<-lapply(p_vect, function(p) nlm(f=function(Y) f(Y=Y, x=x, Max=Max, param=param, a=a, b=b), p=p, iterlim=10000)) # 3 fois moins long qu'un seul DEoptim
          res_nlminb<-lapply(p_vect, function(p) nlminb(objective=function(Y) f(Y=Y, x=x, Min=Min, Max=Max, param=param, a=a, b=b), start=p, lower=Min, upper=Max)) # aussi long qu'un seul Deoptim
          res_nlm_min<-unlist(lapply(res_nlminb, function(x) x$objective))
          Y<-res_nlminb[[which(res_nlm_min==min(res_nlm_min))[1]]]$par
        }
        return(Y)
      }
    }
  }
  else 
    stop("please specify interaction model")
  if (multicore){
    C_mat_list<-split(unique(C_mat), cut(1:(dim(unique(C_mat))[1]), breaks=dim(unique(C_mat))[1]))
    res_CA_unique<-unlist(mclapply(C_mat_list, function(x) Y_f(x, Min=Min, Max=Max, param=param, a=a, b=b), mc.cores=mc.cores))
    res_CA<-rep(NA, dim(unique(C_mat))[1])
    for (i in 1:(dim(unique(C_mat))[1]))
      res_CA[which((C_mat[,1]==unique(C_mat)[i,1])&(C_mat[,2]==unique(C_mat)[i,2]))]<-res_CA_unique[i]
  }
  else {
    res_CA_unique<-apply(unique(C_mat),1, function(x) Y_f(x, Min=Min, Max=Max, param=param, a=a, b=b))
    res_CA<-rep(NA, dim(unique(C_mat))[1])
    for (i in 1:(dim(unique(C_mat))[1]))
      res_CA[which((C_mat[,1]==unique(C_mat)[i,1])&(C_mat[,2]==unique(C_mat)[i,2]))]<-res_CA_unique[i]
  }
  return(res_CA)
}


CA_complete2_RSS_Min_modified_nlm<-function(C_mat, Response, Min, Max, Slopes, Ec50s, a=0, b=0, interact="none", multicore=FALSE, mc.cores=4){
  if (length(Response)!=(dim(C_mat)[1]))
    stop("Should be as many responses as there are conditions")
  res_CA<-CA_complete2_Min_modified_nlm(C_mat=C_mat, Min=Min, Max=Max, Slopes=Slopes, Ec50s=Ec50s, a=a, b=b, interact=interact, multicore=multicore, mc.cores=mc.cores)
  res_CA_RSS<-sum((res_CA-Response)^2)
  return(res_CA_RSS)
}


CA_complete2_fit_Min_modified_nlm_speed<-function(C_mat, Response, upper=NULL, lower=NULL, start=NULL, interact="none", identical_slopes=FALSE, iter=500, multicore=FALSE, mc.cores=4, DEoptim=FALSE){
  if (DEoptim)
    require(DEoptim)
  else
    require(dfoptim)
  if (any(C_mat==0))
    mean_C_mat<-exp(apply(log(C_mat[-which(C_mat==0, arr.ind=TRUE)[,1],]),2,mean))
  else
    mean_C_mat<-exp(apply(log(C_mat),2,mean))
  if (!identical_slopes){
    if (interact=="none"){
      if (missing(upper))
        upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10)
      if (missing(lower))
        lower=c(min(Response)/5,0,-100,-100,0,0)
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], Max=res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[4]), Ec50s=c(res_CA_DEoptim$bestmem[5], res_CA_DEoptim$bestmem[6]), RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), RSS=res_CA_nmk$value))
      }
    }
    if (interact=="SA"){
      if (missing(upper))
        upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10, 20)  # the intervals for a and b must be larger beacuse they can compensate each other
      if (missing(lower))
        lower=c(min(Response)/5,0,-100,-100,0,0, -20)
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], Max=res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[4]), Ec50s=c(res_CA_DEoptim$bestmem[5], res_CA_DEoptim$bestmem[6]), a=res_CA_DEoptim$bestmem[7], RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), a=res_CA_nmk$par[7], RSS=res_CA_nmk$value))
      }
    }
    if (interact=="DR"){
      if (missing(upper))
        upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10, 20, rep(30, (dim(C_mat)[2]-1)))  # the intervals for a and b must be larger beacuse they can compensate each other
      if (missing(lower))
        lower=c(min(Response)/5, 0,-100,-100,0,0, -20, rep(-30, (dim(C_mat)[2]-1)))
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], b=x[8:length(x)], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], Max=res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[4]), Ec50s=c(res_CA_DEoptim$bestmem[5], res_CA_DEoptim$bestmem[6]), a=res_CA_DEoptim$bestmem[7], b=res_CA_DEoptim$bestmem[8: (8+(dim(C_mat)[2]-1)-1)], RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0,rep(0, (dim(C_mat)[2]-1)))
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], b=x[8:length(x)], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), a=res_CA_nmk$par[7], b=res_CA_nmk$par[8: (8+(dim(C_mat)[2]-1)-1)], RSS=res_CA_nmk$value))
      }      
    }
    if (interact=="DL"){
      if (missing(upper))
        upper=c(max(Response), max(Response)*5, 0,0, mean_C_mat*10, 20,30)
      if (missing(lower))
        lower=c(min(Response)/5, 0,-100,-100,0,0, -20, -30)
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], b=x[8], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], Max=res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[4]), Ec50s=c(res_CA_DEoptim$bestmem[5], res_CA_DEoptim$bestmem[6]), a=res_CA_DEoptim$bestmem[7], b=res_CA_DEoptim$bestmem[8], RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[4]), Ec50s=c(x[5], x[6]), a=x[7], b=x[8], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3:4]), Ec50s=c(res_CA_nmk$par[5:6]), a=res_CA_nmk$par[7], b=res_CA_nmk$par[8], RSS=res_CA_nmk$value))
      }      
    }
  }
  else {
    if (interact=="none"){
      if (missing(upper))
        upper=c(max(Response), max(Response)*5, 0, mean_C_mat*10)
      if (missing(lower))
        lower=c(min(Response)/5, 0,-100,0,0)
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[3]), Ec50s=c(res_CA_DEoptim$bestmem[4], res_CA_DEoptim$bestmem[5]), RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3],res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), RSS=res_CA_nmk$value))
      }
    }
    if (interact=="SA"){
      if (missing(upper))
        upper=c(max(Response), max(Response)*5, 0, mean_C_mat*10, 20)  # the intervals for a and b must be larger beacuse they can compensate each other
      if (missing(lower))
        lower=c(min(Response)/5, 0,-100,0,0, -20)
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], Max=res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[3]), Ec50s=c(res_CA_DEoptim$bestmem[4], res_CA_DEoptim$bestmem[5]), a=res_CA_DEoptim$bestmem[6], RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3],res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), a=res_CA_nmk$par[6], RSS=res_CA_nmk$value))
      }
    }
    if (interact=="DR"){
      if (missing(upper))
        upper=c(max(Response), max(Response)*5, 0,mean_C_mat*10, 20, rep(30, (dim(C_mat)[2]-1)))  # the intervals for a and b must be larger beacuse they can compensate each other
      if (missing(lower))
        lower=c(min(Response)/5, 0,-100,0,0, -20, rep(-30, (dim(C_mat)[2]-1)))
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], b=x[7:length(x)], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], Max=res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[3]), Ec50s=c(res_CA_DEoptim$bestmem[4], res_CA_DEoptim$bestmem[5]), a=res_CA_DEoptim$bestmem[6], b=res_CA_DEoptim$bestmem[7: (7+(dim(C_mat)[2]-1)-1)], RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0, rep(0, (dim(C_mat)[2]-1)))
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], b=x[7:length(x)], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3],res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), a=res_CA_nmk$par[6], b=res_CA_nmk$par[7: (7+(dim(C_mat)[2]-1)-1)], RSS=res_CA_nmk$value))
      }
    }
    if (interact=="DL"){
      if (missing(upper))
        upper=c(max(Response),max(Response)*5, 0,mean_C_mat*10, 20,30)
      if (missing(lower))
        lower=c(min(Response)/5,0,-100,0,0, -20, -30)
      if (DEoptim){
        res_CA_DEoptim<-DEoptim(fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], b=x[7], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control = DEoptim.control(iter=iter, trace=TRUE, NP=10*length(lower)))$optim
        return(list(Min=res_CA_DEoptim$bestmem[1], Max=res_CA_DEoptim$bestmem[2], Slopes=c(res_CA_DEoptim$bestmem[3],res_CA_DEoptim$bestmem[3]), Ec50s=c(res_CA_DEoptim$bestmem[4], res_CA_DEoptim$bestmem[5]), a=res_CA_DEoptim$bestmem[6], b=res_CA_DEoptim$bestmem[7], RSS=res_CA_DEoptim$bestval))
      }
      else {
        if (missing(start))
          start=c(min(Response), max(Response),-1,-1,mean_C_mat,0,0)
        res_CA_nmk<-nmkb(par=start, fn=function(x){
          CA_complete2_RSS_Min_modified_nlm(C_mat=C_mat, Response=Response, Min=x[1], Max=x[2], Slopes=c(x[3], x[3]), Ec50s=c(x[4], x[5]), a=x[6], b=x[7], interact=interact, multicore=multicore, mc.cores=mc.cores)
        }, upper=upper, lower=lower, control=list(tol=10^-6))
        return(list(Min=res_CA_nmk$par[1], Max=res_CA_nmk$par[2], Slopes=c(res_CA_nmk$par[3],res_CA_nmk$par[3]), Ec50s=c(res_CA_nmk$par[4:5]), a=res_CA_nmk$par[6], b=res_CA_nmk$par[7], RSS=res_CA_nmk$value))
      }
    }
  }
}





modelFit_CA<-function(anova_model, non_linear_model, nb_param_non_linear){
  SS_anova<-summary(anova_model)[[1]][2,2]
  df_diff<-length(anova_model$fitted.values)-nb_param_non_linear-anova_model$df.residual
  F_value<-(non_linear_model$RSS-SS_anova)/df_diff/(SS_anova/(anova_model$df.residual))  
  p_value<-pf(F_value, df_diff, anova_model$df.residual, lower.tail = FALSE) 
  return(data.frame(F=F_value, p=p_value, df_nl_model=nb_param_non_linear, df_anova_resid=anova_model$df.residual, df_diff) )
}
model_comp<-function(non_linear_model_1, non_linear_model_2, nb_param_non_linear_1, nb_param_non_linear_2, dim_data){
  df_diff<- nb_param_non_linear_1-nb_param_non_linear_2
  F_value<-(non_linear_model_2$RSS-non_linear_model_1$RSS)/df_diff/(non_linear_model_1$RSS/(dim_data-nb_param_non_linear_1))  
  p_value<-pf(F_value, df_diff, dim_data-nb_param_non_linear_1, lower.tail = FALSE) 
  return(data.frame(F=F_value, p=p_value, df_nl_model_1=nb_param_non_linear_1, df_nl_model_2=nb_param_non_linear_2, df_diff) )
}
